# Intra-year growth model for oyster shell

# Import data

# Input: CSV file with digitized trace of oyster (annual) growth layers
# Data is formatted in X and Y columns of arbitrary coordinate system
# Traced shell should be in horizontal orientation with the umbo on the left and the longest extend of the shell on the right (traced shell scans can be mirrored to accomodate this)
# A horizontal baseline (constant X-values) with X-values extending beyond the reach of the shell's cross section is needed as reference
# Make sure extreme X values in both directions along the shell are represented in the baseline
# Input file should contain this baseline's X and Y columns, then the X and Y values of the bottom of the shell, the annual growth line traces of all years in chronological order, then the trace of the top
# Top and bottom lines should meet on both sides of the shell and therefore start and end on the same coordinates
# All lines must be sorted by X-value in ascending order
# All sets of X and Y columns are separated by empty columns

setwd("C://Users//Niels//Dropbox//Research//PhD//Bivalves//B11B//B11 growth//R model")
raw_data<-read.csv("input.csv", sep=";")

# Read data and convert to a common x-axis and coordinate system to work with

Oyster_Convert_cross_section<-function(raw_data, max_length, Xstep=0.1){
    # Convert data to numeric data and remove NA-values for future calculation
    raw_data<-data.frame(lapply(raw_data, as.character), stringsAsFactors=FALSE)
    raw_data<-data.frame(lapply(raw_data, as.numeric))
    raw_data[is.na(raw_data)]<-0
    baseline<-raw_data[,c(1,2)]
    raw_data<-rbind(raw_data,rep(0,length(raw_data[1,])))

    # Adjust the coordinate system to actual mm scale by setting the length of the reference baseline to the actual length of the shell (measured independently)
    lengthfactor<-(max_length / (baseline[which(baseline[,1]==max(baseline[,1]))-1,1]-baseline[which(baseline[,1]==-1)+1,1]))
    lines_mm<-raw_data*lengthfactor

    # Data cleaning
    # In order to set all data to a common x-scale, returning x-values have to be removed to facilitate linear interpolation
    # Data cleaning is done by looping through the X-values and only allowing X-values that are equal to 0 or bigger than the previous value
    # This is done for all XY-records in the input file
    # A new data frame is built with the cleaned data
    lines_mm1<-lines_mm
    for(i in seq(1,ncol(lines_mm1),3)){
        x<-vector()
        y<-vector()
        for(j in 1:(length(lines_mm1[,1]))){
            if(lines_mm1[j,i]==0){
                x<-append(x,lines_mm1[j,i])
                y<-append(y,lines_mm1[j,i+1])
            }
            else if(lines_mm1[j-1,i]==0 | is.na(lines_mm1[j-1,i])){
                x<-append(x,lines_mm1[j,i])
                y<-append(y,lines_mm1[j,i+1])
            }
            else if(lines_mm1[j,i]>lines_mm1[j-1,i]){
                x<-append(x,lines_mm1[j,i])
                y<-append(y,lines_mm1[j,i+1])
            }
        }
        x<-append(x,rep(0,length(lines_mm1[,i])-length(x)))
        y<-append(y,rep(0,length(lines_mm1[,i])-length(y)))
        lines_mm1[,i]<-x
        lines_mm1[,i+1]<-y
    }

    # Linear interpolation
    # Data is assigned a common X-axis by linear interpolation between the data points in the raw data
    # Make sure all x-columns are sorted in ascending order!
    
    # Maximum X-value of the reference baseline is found and rounded up to the next 100 mm
    Xmax<-max(lines_mm[,seq(1,ncol(lines_mm),3)])
    X<-round(Xmax/100+0.5,0)*100
    # A new X-axis is created with Xstep in mm (recommended default: 0.1 mm) 
    lines_lin<-as.data.frame(seq(0,X,Xstep))
    lines_mm2<-lines_mm1
    # All XY-records are linearly interpolated to find values for every X of the new scale 
    for(i in seq(1,ncol(lines_mm2),3)){
        yvect<-vector()
        for(j in lines_lin[,1]){
            close <- which.min(abs(lines_mm2[,i]-j))
            if(lines_mm2[close,i]==j){maxp<-close; minp<-close}
            else {if(lines_mm2[close,i]>j){maxp<-close; minp<-close-1}
                else {minp<-close; maxp<-close+1}
            }
            if(lines_mm2[minp,i+1]==0 | lines_mm2[maxp,i+1]==0) {
                y <- 0
                }
            else {y = lines_mm2[minp,i+1] + ((lines_mm2[maxp,i+1]-lines_mm2[minp,i+1]) / (lines_mm2[maxp,i]-lines_mm2[minp,i])) * (j - lines_mm2[minp,i])
                }
            yvect<-append(yvect,y)
        }
        yvect<-append(yvect,rep(0,length(lines_lin[,1])-length(yvect)))
        # New data frame is created for the interpolated data
        lines_lin<-cbind(lines_lin,yvect)
    }
    colnames(lines_lin)<-c("X",colnames(lines_mm2[,seq(1,ncol(lines_mm2),3)]))

    # New Y-scale with 0 on baseline and positive values upward
    # Subtract Y-values from baseline value to make y-axis point upward (positive values towards higher points in the shell)

    lines_lin2<-lines_lin
    for(i in 1:length(lines_lin[,1])){
        for(j in 2:length(lines_lin[1,])){
            if(is.nan(lines_lin[i,j])){
                lines_lin[i,j]<-lines_lin[i-1,j]
                }
            if(lines_lin[i,j]==0){
                lines_lin2[i,j]<-0
                }
            else {lines_lin2[i,j]<-lines_lin[i,2]-lines_lin[i,j]}
        }
    }

    # Extend growth lines to maximum extend of shell
    # Add values from top and bottom to layer if (x-)extend of shell is less than total reach (in x direction) of the shell to make all layers equally long
    # This equals the length of all growth lines and allows direct addition/subtraction of values

    # Create new data frame
    lines_lin3<-lines_lin2
    # Find first and last values of top and bottom lines
    firstb<-which(lines_lin3[,3]!=0)[1]
    lastb<-which(lines_lin3[,3]!=0)[length(which(lines_lin3[,3]!=0))]
    fistt<-which(lines_lin3[,length(lines_lin3[1,])]!=0)[1]
    lastt<-which(lines_lin3[,length(lines_lin3[1,])]!=0)[length(which(lines_lin3[,length(lines_lin3[1,])]!=0))]
    
    for(i in 3:length(lines_lin3[1,])){
        cat(paste("Converting increment number ",(i-3)),"\r")
        # Find first and last values of yearly growth increments
        first<-which(lines_lin3[,i]!=0)[1]
        last<-which(lines_lin3[,i]!=0)[length(which(lines_lin3[,i]!=0))]
        # Find out if edges of growth lines end in top or bottom of shell and add values from top or bottom lines accordingly 
        for(j in 1:length(lines_lin3[,i])){
            if(j < first) {
                if (abs(lines_lin3[first,i]-lines_lin3[first,3]) < abs(lines_lin3[first,i]-lines_lin3[first,length(lines_lin3[1,])])){
                    lines_lin3[j,i]<-lines_lin3[j,3]
                    }
                else {lines_lin3[j,i]<-lines_lin3[j,length(lines_lin3[1,])]}
                }
            else if (j > last) {
                if (abs(lines_lin3[last,i]-lines_lin3[last,3]) < abs(lines_lin3[last,i]-lines_lin3[last,length(lines_lin3[1,])])){
                    lines_lin3[j,i]<-lines_lin3[j,3]
                    }
                else {lines_lin3[j,i]<-lines_lin3[j,length(lines_lin3[1,])]}
                }
            else {lines_lin3[j,i]<-lines_lin3[j,i]}
        }
    }
    # Export both raw growth line data (year trace) and growth line data with top and bottom values added (cross section)
    Llist<-list(lines_lin3, lines_lin2, lengthfactor)
    return(Llist)
}

# Plot results of cross section

Oyster_plot_cross_section<-function(cross_section){
    x11();
    plot(cross_section[,c(1,3)], type = "l");
    for(i in 3:(length(cross_section[1,])-1)){
        lines(cross_section[,c(1,i)])
    }
}

# Create yearly area matrix of layers
# Calculate area between yearly increments and cumulative area of yearly increments 

Oyster_yearly_matrix<-function(cross_section){
    year<-vector()
    yearly_area<-vector()
    for(i in 4:length(cross_section[1,])){
        area<-0
        for (j in 1:length(cross_section[,1])){
            area<-area + 0.1*(cross_section[j,i-1] - cross_section[j,i])
        }
        yearly_area<-append(yearly_area,area)
        year<-append(year,i-3)
    }
    year<-append(0,year)
    yearly_area<-append(0,yearly_area)
    yearly_cumarea<-cumsum(yearly_area)
    yearly<-cbind(year, yearly_area, yearly_cumarea)
    x11(); plot(yearly, type = "l")
    return(yearly)
}

# Calculate and plot mean shell thickness through time
# Calculate the mean vertical distance between two shell increments and add to yearly matrix

Oyster_Shell_thickness<-function(yearly_matrix, cross_section){
    av_thickness<-0
    for(i in 4:length(cross_section[1,])){
        av_thickness<-append(av_thickness, mean(cross_section[,3]-cross_section[,i]))
    }
    yearly<-cbind(yearly_matrix, av_thickness)
    return(yearly)
    x11(); plot(yearly[,c(1,4)])
}

# Calculate shell length through time

Oyster_Shell_length<-function(yearly_matrix, cross_section){
    # Create vectors of coordinates of first (leftmost) and last (rightmost) points on yearly increment and of shell length
    p1x<-0
    p1y<-0
    p2x<-0
    p2y<-0
    shell_length<-as.vector(0)
    for(i in 4:length(cross_section[1,])){
        # Find left and right points of shell at the time of the increment
        D<-which(c(FALSE,(cross_section[,i]-cross_section[,3])==0)!=c((cross_section[,i]-cross_section[,3])==0,FALSE))
        first<-D[2]
        last<-D[3]-1
        firstX<-cross_section[first,1]
        firstY<-cross_section[first,i]
        lastX<-cross_section[last,1]
        lastY<-cross_section[last,i]
        
        # Correct when points from earlier years result in greater shell lengths (material of earlier years is of course not removed from the shell) 
        if(p1x[length(p1x)]==0 | firstX<=p1x[length(p1x)]){
            p1x<-append(p1x,firstX)
            p1y<-append(p1y,firstY)
        } else {
            p1x<-append(p1x,p1x[length(p1x)])
            p1y<-append(p1y,p1y[length(p1x)])
        }
        if(p2x[length(p2x)]==0 | lastX>=p2x[length(p2x)]){
            p2x<-append(p2x,lastX)
            p2y<-append(p2y,lastY)
        } else {
            p2x<-append(p2x,p2x[length(p2x)])
            p2y<-append(p2y,p2y[length(p2x)])
        }
        # Find shell length by Pythogorian Theorem between the two extreme points on the shell edges (P1 and P2)
        shell_length<-append(shell_length, ((p2x[i-3]-p1x[i-3])^2 + (p2y[i-3]-p1y[i-3])^2)^(0.5))
        # Set point of growth onset to leftmost edge of increment 1 (instead of point [0,0])
        p1x[1]<-p1x[2]
        p2x[1]<-p1x[2]
        p1y[1]<-p1y[2]
        p2y[1]<-p1y[2]
    }
    # Add all data to the yearly matrix
    yearly<-cbind(yearly_matrix,p1x,p1y,p2x,p2y,shell_length)
    return(yearly)
}

# New yearly table through interpolation
# Create a data frame with subyearly growth layers through interpolation of the cross section data 

Oyster_Increment_cross_section<-function(yearly_matrix, cross_section, increment=100, Xstep=0.1){

    # Create a new t-axis with a subyearly time-scale
    incr<-increment
    years<-vector()
    for(i in 0:(length(cross_section[1,])-4)){
        for(tin in 0:(incr-1)){
            years<-append(years,i+(tin/incr))
        }
    }
    yearlyi<-years

    # Calculate increment layers through interpolation of X and Y values between yearly growth lines

    # Create new data frame for incremental cross section
    IncG<-cross_section[,1]
    # Create storage vectors for start and endpoints of subincrements
    p1xs<-vector()
    p2xs<-vector()
    for(i in 1:(length(yearly_matrix[,1])-1)){
        cat(paste("Interpolating increment number ",i),"\r")
        for(tin in 0:(incr-1)){
            # Find the X value of the first and last point on the subyearly inremental line
            p1xtin<-yearly_matrix[i,5]+(yearly_matrix[i+1,5]-yearly_matrix[i,5])/incr*(tin)
            p2xtin<-yearly_matrix[i,7]+(yearly_matrix[i+1,7]-yearly_matrix[i,7])/incr*(tin)
            p1xs<-append(p1xs,p1xtin)
            p2xs<-append(p2xs,p2xtin)

            # Calculate the distance between the subincrement and the top of the shell
            Dinc<-(cross_section[,i+2]-cross_section[,i+3])/incr*tin
            Dinc<-c(rep(0,round(p1xtin/Xstep+0.5,0)),Dinc[(round(p1xtin/Xstep+0.5,0)+1):(round(p2xtin/Xstep-0.5,0)+1)],rep(0,length(Dinc)-(round(p2xtin/Xstep-0.5,0)+1)))
            if(length(Dinc)>length(cross_section[,1])){
                Dinc<-Dinc[1:length(cross_section[,1])]
            }

            # Calculate Y values of the subincrement and add to matrix
            Inc<-cross_section[,i+2]-Dinc
            IncG<-cbind(IncG,Inc)
        }
    }
    # Store in IncG
    IncG<-IncG[,-1]
    colnames(IncG)<-as.numeric(yearlyi)
    rownames(IncG)<-cross_section[,1]
    # Store subyearly data in matrix
    subyearly_matrix<-cbind(yearlyi,p1xs,p2xs)
    Lsub<-list(IncG, subyearly_matrix)
    return(Lsub)
}

IncG<-data.frame(Lsub[1])
colnames(IncG)<-subyearly_matrix[,1]
subyearly_matrix<-data.frame(Lsub[2])

Oyster_plot_Inc_CS<-function(IncG){
    incr<-1/(as.numeric(colnames(IncG)[2])-as.numeric(colnames(IncG)[1]))
    x11(); plot(rownames(IncG),IncG[,1], type = "l");
    for(i in 1:(length(IncG[1,])/incr)){
        lines(rownames(IncG),IncG[,i*incr])
        lines(rownames(IncG),IncG[,(i-0.5)*incr],col="100")
    }
}

Oyster_plot_Inc_fill<-function(IncG){
    IncGt<-IncG
    IncGt<-IncGt[rowSums(IncGt[,-1])>0,]
    incr<-1/(as.numeric(colnames(IncG)[2]))
    x11(width=10,height=10)
    plot(rownames(IncGt),IncGt[,1], type = "l", col=NA, ylim = c(0,max(IncGt[,1])+1))
    lines(rownames(IncGt),IncGt[,1], col="black",lwd=3)
    palette<-heat.colors(length(colnames(IncGt)))
    for(i in 2:length(IncGt[1,])){
        polygon(c(rownames(IncGt),rev(rownames(IncGt))),c(IncGt[,i],rev(IncGt[,i-1])),border=NA,col=palette[i])
        if(i%%incr==0){
            lines(rownames(IncGt),IncGt[,i], col="black",lwd=3)
        }
    }
    lines(rownames(IncGt),IncGt[,length(IncGt[1,])], col="black",lwd=3)
}

# Calculate layer surface area changing through time

Oyster_Subyearly_area<-function(IncG, subyearly_matrix, Xstep){
    areaY<-0
    areaC<-0
    # Area between two increments and cumulative area of cross section is calculated
    for(t in 1:(length(IncG[1,])-1)){
        cat(paste("Calculating area for increment: ",t),"\r")
        areaY<-append(areaY,sum(IncG[,t]-IncG[,t+1])*Xstep)
    }
    areaC<-cumsum(areaY)
    subyearly_matrix<-cbind(subyearly_matrix,areaY,areaC)
    subyearly_matrix[length(subyearly_matrix[,1]),3]<-subyearly_matrix[length(subyearly_matrix[,1])-1,3]
    x11(); plot(subyearly_matrix[,c(1,5)], type = "l")
    return(subyearly_matrix)
}

# Calculate shell length through time

Oyster_Suby_shell_length<-function(subyearly_matrix, IncG, Xstep){
    print("Calculating subincremental shell length")
    # Create matrices for first (leftmost) and last (rightmost) values of increment lines
    p1x<-subyearly_matrix[,2]
    p2x<-subyearly_matrix[,3]
    firstl<-round(p1x/Xstep+0.5,0)+1
    lastl<-round(p2x/Xstep-0.5,0)+1
    p1y<-0
    p2y<-0
    shell_length<-0
    for(t in 1:(length(IncG[1,])-1)){
        p1y<-append(p1y,IncG[firstl[t],t])
        p2y<-append(p2y,IncG[lastl[t],t])
        # Find shell length by Pythogorian Theorem between the two extreme points on the shell edges (P1 and P2)
        L<-sqrt((p2x[t]-p1x[t])^2+(p2y[t]-p1y[t])^2)
        if(L<shell_length[length(shell_length)]){
            L<-shell_length[length(shell_length)]
        }
        shell_length<-append(shell_length,L)
    }
    subyearly_matrix<-cbind(subyearly_matrix,p1y,p2y,shell_length,firstl,lastl)
    return(subyearly_matrix)
}

# Calculate mean shell thickness through time based on X axis

Oyster_Subj_av_thickness_X<-function(IncG,subyearly_matrix){
    print("Calculating subincremental shell thickness")
    IncT<-abs(IncG-IncG[,1])
    IncT[IncT==0]<-NA
    av_thickness<-colMeans(IncT,na.rm=TRUE)
    av_thickness[is.na(av_thickness)]<-0
    subyearly_matrix<-cbind(subyearly_matrix, av_thickness)
    return(subyearly_matrix)
}

# Calculate mean shell thickness through time based on shell length axis

Oyster_Subj_av_thickness<-function(subyearly_matrix){
    print("Calculating subincremental shell thickness")
    av_thickness<-subyearly_matrix[,5]/subyearly_matrix[,8]
    av_thickness[which(is.na(av_thickness))]<-0
    av_thickness[which(av_thickness=="Inf")]<-0
    subyearly_matrix<-cbind(subyearly_matrix, av_thickness)
    return(subyearly_matrix)
}

# Volume of shell during the time of deposition of the growth increment is calculated by assuming ellipse-shaped shell base
# Calculate base ellipse (oval) parameters
# Length (a) and Width (b) of base ellipse of each growth increment is calculated

Oyster_Oval_parameters<-function(subyearly_matrix, IncG, max_shell_length, max_shell_width){
    # r_oval is the ratio between length and width of the ellips, which is fixed for the shell based on the measured values of shell length and maximum width
    r_oval<-max_shell_width/max_shell_length
    L_oval_acc<-vector()
    W_oval<-vector()
    a_oval<-vector()
    b_oval<-vector()
    for(t in 1:(length(IncG[1,]))){
        # L_oval = maximum X-extent of the shell
        #        = p2x - p1x
        # W_oval = maximum Y-extent of the shell
        #        = r_oval * L_oval
        # a = 0.5 * L_oval
        # b = 0.5 * W_oval
        L_oval_acc<-append(L_oval_acc,(subyearly_matrix[t,3]-subyearly_matrix[t,2]))
        W_oval<-append(W_oval,r_oval*(subyearly_matrix[t,3]-subyearly_matrix[t,2]))
        a_oval<-append(a_oval,0.5*r_oval*(subyearly_matrix[t,3]-subyearly_matrix[t,2]))
        b_oval<-append(b_oval,0.5*(subyearly_matrix[t,3]-subyearly_matrix[t,2]))
    }
    subyearly_matrix<-cbind(subyearly_matrix, W_oval, L_oval_acc, a_oval, b_oval)
    return(subyearly_matrix)
}

# Calculate Z-values of ellipses to determine minimum and maximum Z within ellips
# Z-values are the distances from the x-axis of the base ellipse (perpendicular to the cross section)
# These are the maximum extent of the ellipse in the direction of the ellipse's width, and are calculated for every x-value using the formula for ellipse:
# (z/a)^2 + (x/b)^2 = 1
# Which gives: Z = +/- (a/b) * (b^2 - x_el^2)^0.5
# With xel = (x - x0) - b

Oyster_Z_matrices<-function(IncG, subyearly_matrix){
    Z_mat<-as.numeric(rownames(IncG))
    for(t in 1:length(IncG[1,])){
        cat(paste("Calculating Z-values for increment: ",t),"\r")
        a<-subyearly_matrix[t,14]
        b<-subyearly_matrix[t,15]
        x1<-subyearly_matrix[t,2]
        Z<-(a / b) * (b^2 - (as.numeric(rownames(IncG))-b-x1)^2)^(0.5)
        Z_mat<-cbind(Z_mat,Z)        
    }
    Z_mat<-Z_mat[,-1]
    colnames(Z_mat)<-colnames(IncG)
    rownames(Z_mat)<-rownames(IncG)
    Z_mat[is.na(Z_mat)]<-0
    return(Z_mat)
}

# Volume of the oyster at any point in time is calculated by calculating cross sections in the Z-disrection for each X value
# These cross sections are then multiplied by the width of the X-increment (in this case 0.1 mm) and added to obtain total volume
# Volume under the bottom of the shell is subtracted from the volume under the shell increment to obtain net total shell volume
# Volume of individual increments is obtained by calculating the difference between volumes under two subsequent increment layers 

Oyster_Volumes<-function(subyearly_matrix, Z_mat, IncG, Xstep=0.1){
    IncGAnet<-data.frame(matrix(0,ncol=length(IncG[1,]),nrow=length(IncG[,1])))
    colnames(IncGAnet)<-colnames(IncG)
    rownames(IncGAnet)<-rownames(IncG)
    VolC<-vector()
    As2<-vector()
    Y1s2<-vector()
    Ytops2<-vector()
    Y0s2<-vector()
    rs2<-vector()
    Ytopbs2<-vector()
    Y0bs2<-vector()
    rbs2<-vector()
    for(t in 1:length(IncGAnet[1,])){
        cat(paste("Calculating volume for increment: ",t),"\r")
        As<-vector()
        Y1s<-vector()
        Ytops<-vector()
        Y0s<-vector()
        rs<-vector()
        Ytopbs<-vector()
        Y0bs<-vector()
        rbs<-vector()
        for (x in 1:(length(IncGAnet[,1]))){
            # Universal parameters Z and Y1
            # Of which Y1 is the height of the base oval that forms the bottom of the shell under the increment, and Z is the distance from the x-axis to the side of the ellips
            Z<-Z_mat[x,t]
            Y1<-subyearly_matrix[t,6] + ((subyearly_matrix[t,7]-subyearly_matrix[t,6])/(subyearly_matrix[t,3]-subyearly_matrix[t,2])) * (as.numeric(rownames(IncGAnet)[x])-subyearly_matrix[t,2])
            if(is.na(Y1)){Y1<-0}
            # Parameters increment

            # Of which Ytop is the maximum height of the cross section of the shell under the increment, which is found for the X value by linear interpolation in the XY trace of the shell increment
            # Ytop is the Y value calculated in the incremental growth matrix

            # Y0 is the Y value of the center of the virtual circle that connects the top of the shell with the two points at the edge of the ellipse (Z1 and Z)
            # Formula of circle x^2 + y^2 = r^2 with points [-Z,Y1], [0,Ytop], and [Z,Y1], and center of circle = [0, Y0] gives: 
            # (0-0)^2 + (Ytop-Y0)^2 = (-Z-0)^2 + (Y1 - Y0)^2 = (Z- 0)^2 + (Y1 - Y0)^2
            # Y0 = (Ytop^2 - Y1^2 - Z^2) / (2*(Ytop - Y1))
            # r is the radius of said circle
            # r = (x^2 - y^2)^0.5 in point [Z, Y1] and center [0, Y0] gives:
            # r = (Z^2 + (Y1 - Y0)^2)^0.5
            Ytop<-IncG[x,t]
            Y0<-(Ytop^2-Y1^2-Z^2)/(2*(Ytop-Y1))
            r<-(Z^2+(Y1-Y0)^2)^0.5
            # Parameters shell top (b)
            # Ytopb and rb are calculated in the same way as Ytop and r above
            Ytopb<-IncG[x,1]
            Y0b<-(Ytopb^2-Y1^2-Z^2)/(2*(Ytopb-Y1))
            rb<-(Z^2+(Y1-Y0b)^2)^0.5

            # Prevent negative areas if increment go above the shell top
            if(Ytopb<Ytop){Ytop<-Ytopb}

            # Calculate area under shell increment
            # The area of the circle segment that is under the growth increment is found by 
            # subtracting the area of the triangle formed by the center of the circle and points P2[-Z,Y1] and P3[Z,Y1] from that of the corresponding circle sector 
            # A_circle = A_sector - A_triangle
            #          = angle/(2*pi) * pi * r^2 - rsin(angle)*rcos(angle)
            #          = 0.5*angle*r^2 - 0.5*r^2*sin(angle)
            #          = 0.5*r^2*(angle - sin(angle))
            # In which angle is the angle between the lines between the two points P2[-Z,Y1] and P3[Z,Y1] and the center of the circle (Pc)
            # The total area under the shell increment is always equal to the area between the circle segment and the x-axis
            # Area between the shell and the x-axis is not taken into account as the difference between top and shell increment is taken later on

            if(is.na(r)|r==abs(Inf)){
                angle<-0
                r<-0
            }
            else angle<-asin(Z/r)*2
            # If Y1 is higher than the top of the shell, the circle flips and has to be subtracted from the rectangle between Y0 and the x-axis
            if(Y1>Ytop){
                A<-(2*Z*Y1)-0.5*r^2*(angle - sin(angle))
            }
            else{
                # If Y0 is in between Y1 and Ytop, then the area is calculated by the full circle formed by Y0 (center) and Ytop minus the circlesection under Y1
                if(Y0>Y1 & !is.na(Y1) & !is.na(Y0)){
                    A<-r^2*pi-(0.5*r^2*(angle - sin(angle)))+(2*Z*Y1)
                }
                else A<-0.5*r^2*(angle - sin(angle))+(2*Z*Y1)
            }

            # Calculate area under shell top in the same way as the area under the increment was calculated
            if(is.na(rb)|rb==abs(Inf)){
                angleb<-0
                rb<-0
            }
            else angleb<-asin(Z/rb)*2
            if(Y1>Ytopb){
                Ab<-(2*Z*Y1)-0.5*rb^2*(angleb - sin(angleb))
            }
            else{
                # If Y0b is in between Y1 and Ytopb, then the area is calculated by the full circle between Y0b (center) and Ytopb minus the circlesection under Y1
                if(Y0b>Y1 & !is.na(Y1) & !is.na(Y0b)){
                    Ab<-rb^2*pi-(0.5*rb^2*(angleb - sin(angleb)))+(2*Z*Y1)
                }
                else Ab<-0.5*rb^2*(angleb - sin(angleb))+(2*Z*Y1)
            }
            # Prevent cases where increment overlaps with top of shell
            if(Ytop==Ytopb){
                Ab<-A
            }

            # Prevent cases of negative area
            if(Ab<A){
                Ab<-A
            }

            # Calculate area relative to shell top by subtracting area under top of shell from that under the shell (sub)increment
            IncGAnet[x,t]<-Ab-A
            As<-append(As,Ab-A)
            Y1s<-append(Y1s,Y1)
            Ytops<-append(Ytops,Ytop)
            Y0s<-append(Y0s,Y0)
            rs<-append(rs,r)
            Ytopbs<-append(Ytopbs,Ytopb)
            Y0bs<-append(Y0bs,Y0b)
            rbs<-append(rbs,rb)
        }
        # Calculate total volume relative to zero line per time increment by adding up the areas of all perpendicular (YZ)cross sections under the same increment and multiplying by the increment width
        VolC<-append(VolC,sum(As)*Xstep)
        As2<-cbind(As2,As)
        Y1s2<-cbind(Y1s2,Y1s)
        Ytops2<-cbind(Ytops2,Ytops)
        Y0s2<-cbind(Y0s2,Y0s)
        rs2<-cbind(rs2,rs)
        Ytopbs2<-cbind(Ytopbs2,Ytopbs)
        Y0bs2<-cbind(Y0bs2,Y0bs)
        rbs2<-cbind(rbs2,rbs)
    }
    # Calculate volume gain per time increment by subtracting volume value for each increment by its predecessor.
    # WARNING: changes in the slope of Y1 will be amplified into large oscillations of VolI. It is advised to use a smoothing of VolI for further analysis
    VolI<-c(VolC,0)-c(0,VolC)
    VolI<-VolI[-length(VolI)]
    # Prevent negative volume increments
    VolI[VolI<0]<-0
    VolC<-cumsum(VolI)
    subyearly_matrix<-cbind(subyearly_matrix,VolI,VolC)
    diagL<-list(subyearly_matrix,IncGAnet,As2,Y1s2,Ytops2,Y0s2,rs2,Ytopbs2,Y0bs2,rbs2)
    return(diagL)
}

subyearly_matrix<-as.data.frame(diagL[1])
IncGAnet<-as.data.frame(diagL[2])

# Calculate shell weight (g) and weight gain (g/y) through time by multiplying with the density in g/cm3

Oyster_Mass_gain<-function(subyearly_matrix, density=2.71, increment=100){
    WeightC<-subyearly_matrix[,17]*density*0.001
    Growth_rate<-subyearly_matrix[,16]*density*0.001*increment
    subyearly_matrix<-cbind(subyearly_matrix,Growth_rate,WeightC)
    rownames(subyearly_matrix)<-subyearly_matrix[,1]
    return(subyearly_matrix)
}

# Export data

Oyster_Export<-function(subyearly_matrix, name_file){
    require(xlsx)
    write.xlsx(subyearly_matrix,paste(name_file, ".xlsx"))
}

# ----------------------------------------------------------------------------------------------------------------------------------------------------- #

# Oyster BMP model
# Extracts data from phase or greyscale bmp to estimate concentrations of elements through oyster growth

#Import BMP of phase analysis
Oyster_Import_BMP<-function(file_name){
    require(bmp)
    BMP<-read.bmp(file_name)
    return(BMP)
}

# Import TIF of phase analysis
Oyster_import_TIF<-function(file_name){
    require(tiff)
    TIF<-readTIFF(file_name)
    return(TIF)
}

# Import matrix of phase characteristics (concentrations in %)
Oyster_Import_phases<-function(file_name){
    phase<-read.csv(file_name, sep=";")
    phase[,as.numeric(which(sapply(phase,class)=="factor"))]<-data.frame(lapply(phase[,as.numeric(which(sapply(phase,class)=="factor"))], as.character), stringsAsFactors=FALSE)
    return(phase)
}

# Create matrix of phases from BMP
Oyster_phase_matrix_BMP<-function(BMP,phases,exclude_phases){
    phasemat<-matrix(ncol=ncol(BMP),nrow=nrow(BMP))
    for(a in 1:nrow(BMP)){
        cat(paste("Converting row ",a, " of ",nrow(BMP)),"\r")
        for(b in 1:ncol(BMP)){
            # Use pythagorean theorem to find closest match of phase (prevents bug in case of distorted pixels)
            phasemat[a,b]<-phases[which.min(sqrt((BMP[a,b,1]-phases[,3])^2+(BMP[a,b,2]-phases[,4])^2+(BMP[a,b,3]-phases[,5])^2)),2]
        }
    }
    # Get rid of factor columns in phasemat
    return(phasemat)
}

Oyster_phase_matrix_TIF<-function(TIF,phases){
    TIF<-round(TIF*256,0)
    phasemat<-matrix(F, nrow=nrow(TIF), ncol=ncol(TIF))
    # Assign matching phases
    for(p in 1:nrow(phases)){
        phasemat[which((TIF[,,1]==phases[p,3]&TIF[,,2]==phases[p,4]&TIF[,,3]==phases[p,5])==T)]<-as.character(phases[p,2])
    }
    # Assign closest phases
    phasemat[phasemat==F]<-phasemat[which(phasemat==F)[diff(which(phasemat==F))!=1]+1]
    return(phasemat)
}

# Calculate relative representation of phases in shell and corresponding compositions
Oyster_phase_stats<-function(phasemat,phases){
    fraction<-vector()
    pixels<-vector()
    for(i in 1:length(phases[,2])){
        pixels<-append(pixels,length(which(phasemat==phases[i,2])))
        fraction<-append(fraction,length(which(phasemat==phases[i,2]))/length(phasemat))
    }
    phase_stat<-cbind(phases[,2],pixels,fraction,phases[,6:length(phases[1,])])
    
    phase_stat[,2:length(phase_stat[1,])]<-lapply(phase_stat[,2:length(phase_stat[1,])],as.numeric)
    phase_stat[,1]<-levels(phase_stat[,1])
    Bulkcomp<-as.vector(c("Bulk composition",as.numeric(sum(phase_stat[,2])),as.numeric(sum(phase_stat[,3])),as.numeric(colSums(phase_stat[,3]*phase_stat[,4:27]))))
    phase_stat<-rbind(phase_stat, Bulkcomp)

    return(phase_stat)
}

# Calculate matrix of amount of pixels of each phase in growth increments
# Pixel size is in mm
Oyster_suby_phases<-function(IncG, phasemat, pixelsize, phases){
    phase_mat<-vector()
    for(t in 1:(length(IncG[1,])-1)){
        tph<-as.vector(rep(0,length(phases[,1])))
        cat(paste("Calculating increment: ",t),"\r")
        for(x in 1:length(IncG[,1])){
            xph<-vector()
            y2<-IncG[x,t]/pixelsize
            y1<-IncG[x,t+1]/pixelsize
            x1<-as.numeric(rownames(IncG)[x])/pixelsize
            x2<-as.numeric(rownames(IncG)[x])/pixelsize
            if(x2>length(phasemat[1,])){
                xph<-rep(0,length(phases[,1]))
            }
            else{
                for(p in 1:length(phases[,1])){
                    xph<-append(xph,length(which(phasemat[round(y1-0.5,0):round(y2+0.5,0),round(x1-0.5,0):round(x2+0.5,0)]==phases[p,2])))
                }
            }
            tph<-tph+xph
        }
        phase_mat<-cbind(phase_mat,tph)
    }
    phase_mat<-cbind(rep(0,length(phases[,1])),phase_mat)
    colnames(phase_mat)<-colnames(IncG)
    return(phase_mat)
}

# Calculate elemental concentrations changing through time (in %) based on phase representation
# Exclude phases that represent the background (outside of the shell)
Oyster_el_time<-function(phase_mat, phases, exclude_phases){
    # Exclude non-relevant phases
    phase_mat_f<-phase_mat[which(!phases[,2] %in% exclude_phases),]
    phases_f<-phases[which(!phases[,2] %in% exclude_phases),]
    # Calculate relative phase contributions
    phase_mat_f<-t(t(phase_mat_f)/as.vector(colSums(phase_mat_f)))
    # Remove zero-entries
    phase_mat_f[,which(colSums(phase_mat_f)==0)]<-rep(0,length(phases_f[,1]))
    # Calculate concentrations from relative abundances of elements in phases
    elmat<-phases_f[,6:length(phases_f[1,])]
    el_time<-vector()
    for(t in 1:length(phase_mat_f[1,])){
        els<-as.numeric(colSums(phase_mat_f[,t]*elmat))
        el_time<-cbind(el_time,els)
    }
    el_time[is.na(el_time)]<-0
    colnames(el_time)<-colnames(phase_mat)
    rownames(el_time)<-colnames(elmat)
    return(el_time)
}

# Function for moving average of time series
pma<-function(x,i,n){
	ts<-cbind(x[,1],x[,i])
	ts<-cbind(ts,rep(NA,length(x[,1])))
	for (a in ((n/2+0.5):(length(x[,1])+0.5-(n/2)))) {
		ts[a,3] = mean(ts[(a-(n/2)+1):(a+(n/2)),2],na.rm=T)
		}		
	return(ts)
	}

# Calculate accumulation rates of elements through time
Oyster_accumulation<-function(el_time, subyearly_matrix, increment=100, npma=10){
    WI<-pma(subyearly_matrix,18,npma)
    WI[is.na(WI)]<-0
    # Mass accumulation in grams/inc
    M_el_mat<-t(el_time)/increment*WI[,3]
    # Cumulative mass of elements in grams
    M_el_cum_mat<-apply(t(M_el_mat),1,cumsum)/increment
    return(list(M_el_cum_mat,M_el_mat))
}

Oyster_phase_export<-function(phase_stat, el_time, M_el_mat, M_el_mat_c, name_shell){
    require(xlsx)
    write.xlsx(phase_stat,paste(name_shell, " phase_stat.xlsx"))
    write.xlsx(el_time,paste(name_shell, " el_time.xlsx"))
    write.xlsx(M_el_mat,paste(name_shell, " M_el_mat.xlsx"))
    write.xlsx(M_el_mat_c,paste(name_shell, " M_el_mat_c.xlsx"))
}

# Plot modelled accumulation rates
Oyster_plot_SrFe_divCa_acc<-function(M_el_mat){
    x11()
    plot(as.numeric(colnames(M_el_mat)),M_el_mat[which(rownames(M_el_mat)=="Sr"),]/M_el_mat[which(rownames(M_el_mat)=="Ca"),], type="l")
    lines(as.numeric(colnames(M_el_mat)),M_el_mat[which(rownames(M_el_mat)=="Sr"),]/M_el_mat[which(rownames(M_el_mat)=="Ca"),])
}

# Run the model!
# B11B shell length = 168.156 mm
# B11B phase map length (max_length) = 176.2 mm
# B11B max shell width = 135 mm
# Density oyster shell calcite = 2.679 g/cm3

# As_AA_2 shell length = 32.45 mm
# As_AA_2 map length = 35.46 mm
# As_AA_2 max shell width = 26.05 mm

# As_I_1 shell length = 54.07 mm
# As_I_1 map length = 61.25 mm
# As_I_1 max shell width = 43.41 mm


Oyster_run_sec2<-function(raw_data, image_length, Xstep){

    #Modify data to fit XY-coordinates and to create cross section
    print("Converting traced shell data into cross section")
    Llist<-Oyster_Convert_cross_section(raw_data, image_length, Xstep)
    cross_section<-Llist[[1]]
    year_trace<-Llist[[2]]
    lengthfactor<-Llist[[3]]
    rm(Llist)

    # Plot results of cross section
    print("Plotting cross section")
    Oyster_plot_cross_section(cross_section)

    # Create yearly area matrix of layers
    print("Calculating yearly values for shell volume, length and thickness")
    yearly_matrix<-Oyster_yearly_matrix(cross_section)
    yearly_matrix<-Oyster_Shell_thickness(yearly_matrix, cross_section)
    yearly_matrix<-Oyster_Shell_length(yearly_matrix, cross_section)

    List2<-list(cross_section, yearly_matrix, lengthfactor)
    return(List2)
}

Oyster_run_sec3<-function(cross_section, yearly_matrix, Xstep, increment, Oyster_length, Oyster_width){

    # New yearly table through interpolation
    print("Interpolating cross section data to calculate intra-year growth increments")
    Lsub<-Oyster_Increment_cross_section(yearly_matrix, cross_section, increment, Xstep)
    IncG<-data.frame(Lsub[1])
    subyearly_matrix<-data.frame(Lsub[2])
    colnames(IncG)<-subyearly_matrix[,1]

    # Plot increment cross section
    print("Plotting intra-year growth cross section")
    Oyster_plot_Inc_CS(IncG)
    Oyster_plot_Inc_fill(IncG)

    # Calculate subyearly matrix of parameters
    print("Calculating subyearly values for shell cross section volume, length, thickness and base ellips parameters for total shell volume")
    subyearly_matrix<-Oyster_Subyearly_area(IncG, subyearly_matrix, Xstep)
    subyearly_matrix<-Oyster_Suby_shell_length(subyearly_matrix, IncG, Xstep)
    subyearly_matrix<-Oyster_Subj_av_thickness(subyearly_matrix)
    subyearly_matrix<-Oyster_Oval_parameters(subyearly_matrix, IncG, Oyster_length, Oyster_width)

    List3<-list(IncG, subyearly_matrix)
    return(List3)
}

Oyster_run_sec4<-function(IncG, subyearly_matrix, Xstep = 0.1, increment = 100, density = 2.71){

    # Calculate Z-Values for volume modeling
    print("Calculating Z-values for volume model")
    Z_mat<-Oyster_Z_matrices(IncG, subyearly_matrix)

    # Calculate in subyearly matrix the volumes and mass gain
    print("Calculating volume and growth rate through time")
    diagL<-Oyster_Volumes(subyearly_matrix, Z_mat, IncG, Xstep)
    subyearly_matrix<-as.data.frame(diagL[1])
    IncGAnet<-as.data.frame(diagL[2])

    subyearly_matrix<-Oyster_Mass_gain(subyearly_matrix, density, increment)

    List4<-list(subyearly_matrix, IncGAnet)
    return(List4)
}

Oyster_run_sec5<-function(file, file_ext, phases, exclude_phases){
    
    if(file_ext=="bmp"|file_ext=="BMP"){
        print("Constructing matrix of pixels and their phases")
        phasemat<-Oyster_phase_matrix_BMP(file, phases, exclude_phases)
    }
    else if(file_ext=="tif"|file_ext=="tiff"|file_ext=="TIF"|file_ext=="TIFF"){
        print("Constructing matrix of pixels and their phases")
        phasemat<-Oyster_phase_matrix_TIF(file, phases)
    }

    print("Calculating phase statistics of the entire map")
    phase_stat<-Oyster_phase_stats(phasemat, phases)

    print("Exporting data as list")
    List5<-list(phasemat, phase_stat)
    return(List5)
}

Oyster_run_sec6<-function(phasemat, IncG, pixelsize, phases, exclude_phases, subyearly_matrix, increment, npma){
    
    print("Calculating phase statistics per subyearly growth increment")
    phase_mat<-Oyster_suby_phases(IncG, phasemat, pixelsize, phases)

    print("Calculating concentrations of elements through time using phase data")
    el_time<-Oyster_el_time(phase_mat, phases, exclude_phases)

    print("Calculating accumulation rates based on concentrations through time and mass accumulation of growth model")
    AccL<-Oyster_accumulation(el_time, subyearly_matrix, increment, npma) 
    M_el_mat_c<-as.data.frame(AccL[1])
    M_el_mat<-as.data.frame(AccL[2])

    print("Exporting data as list")
    List6<-list(el_time, M_el_mat, M_el_mat_c)
    return(List6)
}

Oyster_log<-function(name_routine,...){
    name<-as.character(as.list(match.call())[-c(1,2)])
    print(name)
    val<-as.numeric(list(...))
    print(val)
    lst<-list(paste("LOG of parameters of shell growth model ran on ", date(),"\nRoutine : ", name_routine,"\n"))
    for(i in 1:length(val)){
        lst<-append(lst,paste(name[i]," : ",val[i]))
    }
    write.table(lst, file="LOG.txt", sep="\n", col.names=F, row.names=F, quote=F)
}

Oyster_growth_run<-function(raw_data, max_length, Xstep=0.1, increment=100, max_shell_length, max_shell_width, density=2.71, name_file="Oyster_growth_model"){
    Oyster_log("Oyster_growth_run", max_length, Xstep, increment, max_shell_length, max_shell_width, density, name_file)

    List2<-Oyster_run_sec2(raw_data, max_length, Xstep)
    cross_section<-as.data.frame(List2[1])
    yearly_matrix<-as.data.frame(List2[2])
    lengthfactor<-List2[3]
    rm(List2)

    List3<-Oyster_run_sec3(cross_section, yearly_matrix, Xstep, increment, max_shell_length, max_shell_width)
    IncG<-as.data.frame(List3[1])
    subyearly_matrix<-as.data.frame(List3[2])
    rm(List3)

    List4<-Oyster_run_sec4(IncG, subyearly_matrix, Xstep, increment, density)
    subyearly_matrix<-as.data.frame(List4[1])
    IncGAnet<-as.data.frame(List4[2])
    rm(List4)

    # Export data
    print("Exporting (sub)yearly data into XLSX file")
    Oyster_Export(subyearly_matrix, name_file)

    return(subyearly_matrix)
}

Oyster_phase_run<-function(file_name, file_ext, phases_name, IncG, pixelsize, exclude_phases, subyearly_matrix, name_shell, increment=100, npma=10){
    Oyster_log("Oyster_phase_run",file_name, file_ext, phases_name,pixelsize, exclude_phases, name_shell, increment=100, npma=10)
    
    if(file_ext=="bmp"|file_ext=="BMP"){
        BMP<-Oyster_Import_BMP(file_name)
        phases<-Oyster_Import_phases(phases_name)

        List5<-Oyster_run_sec5(BMP, file_ext, phases, exclude_phases)
        phasemat<-as.matrix(as.data.frame(List5[1]))
        phase_stat<-as.data.frame(List5[2])
        rm(List5)
    }
    else if(file_ext=="tif"|file_ext=="tiff"|file_ext=="TIF"|file_ext=="TIFF"){
        TIF<-Oyster_import_TIF(file_name)
        phases<-Oyster_Import_phases(phases_name)

        List5<-Oyster_run_sec5(TIF, file_ext, phases)
        phasemat<-as.matrix(as.data.frame(List5[1]))
        phase_stat<-as.data.frame(List5[2])
        rm(List5)
    }
    else(return("Invalid file extension"))

    List6<-Oyster_run_sec6(phasemat, IncG, pixelsize, phases, subyearly_matrix, increment, npma)
    el_time<-as.data.frame(List6[1])
    M_el_mat<-as.data.frame(List6[2])
    M_el_mat_c<-as.data.frame(List6[3])
    rm(List6)

    Oyster_phase_export(phase_stat, el_time, M_el_mat, M_el_mat_c, name_shell)

    print("Exporting model results into XLSX files")
    output<-list(phase_stat, el_time, M_el_mat, M_el_mat_c)
    return(output)
}

Oyster_combined_run<-function(raw_data, max_length, Xstep=0.1, increment=100, max_shell_length, max_shell_width, density=2.71, name_file="Oyster_growth_model", file_name, file_ext, phases_name, exclude_phases, name_shell, npma=10){
    Oyster_log("Oyster_growth_run", max_length, Xstep, increment, max_shell_length, max_shell_width, density, name_file)

    List2<-Oyster_run_sec2(raw_data, max_length, Xstep)
    cross_section<-as.data.frame(List2[1])
    yearly_matrix<-as.data.frame(List2[2])
    lengthfactor<-List2[3]
    rm(List2)

    List3<-Oyster_run_sec3(cross_section, yearly_matrix, Xstep, increment, max_shell_length, max_shell_width)
    IncG<-as.data.frame(List3[1])
    subyearly_matrix<-as.data.frame(List3[2])
    rm(List3)

    List4<-Oyster_run_sec4(IncG, subyearly_matrix, Xstep, increment, density)
    subyearly_matrix<-as.data.frame(List4[1])
    IncGAnet<-as.data.frame(List4[2])
    rm(List4)

    # Export data
    print("Exporting (sub)yearly data into XLSX file")
    Oyster_Export(subyearly_matrix, name_file)

    Oyster_log("Oyster_phase_run",file_name, file_ext, phases_name, lengthfactor, exclude_phases,name_shell, increment=100, npma=10)
    
    if(file_ext=="bmp"|file_ext=="BMP"){
        BMP<-Oyster_Import_BMP(file_name)
        phases<-Oyster_Import_phases(phases_name)

        List5<-Oyster_run_sec5(BMP, file_ext, phases)
        phasemat<-as.matrix(as.data.frame(List5[1]))
        phase_stat<-as.data.frame(List5[2])
        rm(List5)
    }
    else if(file_ext=="tif"|file_ext=="tiff"|file_ext=="TIF"|file_ext=="TIFF"){
        TIF<-Oyster_import_TIF(file_name)
        phases<-Oyster_Import_phases(phases_name)

        List5<-Oyster_run_sec5(TIF, file_ext, phases)
        phasemat<-as.matrix(as.data.frame(List5[1]))
        phase_stat<-as.data.frame(List5[2])
        rm(List5)
    }
    else(return("Invalid file extension"))

    List6<-Oyster_run_sec6(phasemat, IncG, lengthfactor, phases, subyearly_matrix, increment, npma)
    el_time<-as.data.frame(List6[1])
    M_el_mat<-as.data.frame(List6[2])
    M_el_mat_c<-as.data.frame(List6[3])
    rm(List6)

    Oyster_phase_export(phase_stat, el_time, M_el_mat, M_el_mat_c, name_shell)

    print("Exporting model results into XLSX files")
    output<-list(subyearly_matrix, phase_stat, el_time, M_el_mat, M_el_mat_c)
    return(output)
}